@@ -1,7 +1,5 @@
 #!/usr/bin/env python3
 
-import click
-import click_log
 import glob
 import json
 import logging
@@ -11,13 +9,15 @@ from tempfile import TemporaryFile, NamedTemporaryFile
 from typing import List, Dict, Union
 from zipfile import ZipFile, ZIP_DEFLATED
 
-from fluidtopics.connector.model.metadata import SemanticMetadata
+import click
+import click_log
 
-from .ftmap import FTMap
-from .metadata import FT_ORIGIN_ID, FT_LANG, get_md_metas
-from .errors import MD2FTError
-from .ftsession import FTSession
-from . import __version__ as md2ft_version
+from fluidtopics.connector.model.metadata import SemanticMetadata
+from fluidtopics.markdown import __version__ as md2ft_version
+from fluidtopics.markdown.errors import MD2FTError
+from fluidtopics.markdown.ftmap import FTMap
+from fluidtopics.markdown.ftsession import FTSession
+from fluidtopics.markdown.metadata import FT_ORIGIN_ID, FT_LANG, get_md_metas, MISSING_TITLE
 
 logger = logging.getLogger("fluidtopics.markdown")
 click_log.basic_config(logger)
@@ -44,12 +44,12 @@ def prune_empty_dir(content: Dict[str, dict]) -> Dict[str, dict]:
 
 
 def dir_content(
-    curdir: Path,
-    relto: Path,
-    skip: List[Path],
-    meta_include_list: List[str],
-    implicit_meta: bool = False,
-    dev_mode: bool = False,
+        curdir: Path,
+        relto: Path,
+        skip: List[Path],
+        meta_include_list: List[str],
+        implicit_meta: bool = False,
+        dev_mode: bool = False,
 ) -> Dict[str, dict]:
     """Recursively Collect the directory content starting at curdir"""
     content = {}
@@ -67,10 +67,10 @@ def dir_content(
                 continue
             dirdesc = file / "README.md"
             if dirdesc.exists():
-                dirmetas, with_content = get_md_metas(
-                    dirdesc
+                dirmetas, with_content, heading_title = get_md_metas(
+                    dirdesc,
+                    dev_mode=dev_mode
                 )  # We handle default title for directory differently...
-                dirtitle = dirmetas.get(SemanticMetadata.TITLE, file.name)
                 logger.debug(f"Found directory description in {dirdesc}")
                 skip.append(dirdesc.relative_to(relto))
                 if meta_include_list:
@@ -90,7 +90,7 @@ def dir_content(
                     content[relf_posix] = {
                         "value": {
                             "href": dirdesc.relative_to(relto).as_posix(),
-                            SemanticMetadata.TITLE: dirtitle,
+                            SemanticMetadata.TITLE: heading_title,
                             "metas": dirmetas,
                         },
                         "childs": dir_content(
@@ -107,7 +107,7 @@ def dir_content(
                     content[relf_posix] = {
                         "value": {
                             FT_ORIGIN_ID: f_posix,
-                            SemanticMetadata.TITLE: dirtitle,
+                            SemanticMetadata.TITLE: heading_title,
                             "metas": dirmetas,
                         },
                         "childs": dir_content(
@@ -144,7 +144,8 @@ def dir_content(
                     }
 
         elif file.is_file() and file.match("*.md"):
-            metas, _ = get_md_metas(file, implicit_meta)
+            metas, topic_content, heading_title = get_md_metas(file, implicit_meta, str(file.relative_to(relto)) == 'README.md',
+                                                   dev_mode=dev_mode)
             audience = metas.get("audience", None)
             if meta_include_list:
                 if audience is None:
@@ -157,27 +158,19 @@ def dir_content(
                         f"Skipping {file.relative_to(relto).as_posix()} as audience={audience} not in include_list = {meta_include_list}"
                     )
                     continue
-            if metas.get(SemanticMetadata.TITLE, None) is not None:
+            if topic_content.strip() or relf_posix != 'README.md':
                 content[relf_posix] = {
                     "value": {
                         "href": relf_posix,
-                        SemanticMetadata.TITLE: metas[SemanticMetadata.TITLE],
+                        SemanticMetadata.TITLE: heading_title,
                         "metas": metas,
                     }
                 }
-            else:
-                if dev_mode:
-                    logger.warning(
-                        f"Skipping {file.as_posix()} no ft:title given as meta."
-                    )
-                else:
-                    raise MD2FTError(
-                        f"no meta ft:title or # title found in {file.as_posix()}"
-                    )
     return content
 
 
-def _create_zip_file(output, pinput, image_dir, ftmap_file, collected_files):
+def _create_zip_file(output: str, pinput: Path, image_dir: str, ftmap_file: TemporaryFile, collected_files: List[Path],
+                     dev_mode: bool):
     with ZipFile(output, mode="w", compression=ZIP_DEFLATED) as archive:
         archive.writestr("generated.ftmap", data=ftmap_file.read())
         # NB: we need to use glob.glob and not pathlib.Path.rglob
@@ -200,18 +193,18 @@ def _create_zip_file(output, pinput, image_dir, ftmap_file, collected_files):
                 archive.write(f, arcname=f.relative_to(pinput))
             elif f.relative_to(pinput) in collected_files:
                 # get MD content with stripped-out metadata
-                _, md_content = get_md_metas(f)
+                _, md_content, _ = get_md_metas(f, dev_mode=dev_mode)
                 archive.writestr(str(f.relative_to(pinput)), md_content)
             else:
                 logger.debug(f"{f.relative_to(pinput)} not collected in archive.")
 
 
 def do_collect(
-    input: str,
-    output: str,
-    included_audiences: List[str],
-    implicit_meta: bool,
-    dev_mode: bool = False,
+        input: str,
+        output: str,
+        included_audiences: List[str],
+        implicit_meta: bool,
+        dev_mode: bool = False,
 ):
     """Collect markdown files and create FTML archive with a book FT Map
     built from the metadata found in the markdown files (unless implicit_meta is True)
@@ -220,10 +213,11 @@ def do_collect(
     pinput = Path(input)
     root_readme = list(pinput.glob("README.md"))
     metas = None
+    root_readme_title = MISSING_TITLE
     # We always collect metas from root README is there is some
     if root_readme:
         root_readme = root_readme[0]
-        metas, root_content = get_md_metas(root_readme)
+        metas, root_content, root_readme_title = get_md_metas(root_readme, dev_mode=dev_mode)
     if implicit_meta:
         logger.debug("Implicit meta MODE")
         implicit_metas_values = {
@@ -263,13 +257,13 @@ def do_collect(
     # Clean up empty dirs (i.e. which do not contain md files)
     collected = prune_empty_dir(collected)
     logger.debug(f"collected= {collected}")
-    if SemanticMetadata.TITLE not in metas:
+    if SemanticMetadata.TITLE not in metas and root_readme_title == MISSING_TITLE:
         raise MD2FTError(f"No ft:title in {root_readme}.")
     if FT_ORIGIN_ID not in metas:
         raise MD2FTError(f"No ft:originID in {root_readme}.")
     # Init FTMAP with mandatory prop: ft:title, ft:originID
     ftmap = FTMap(
-        title=metas[SemanticMetadata.TITLE],
+        title=root_readme_title,
         origin_id=metas[FT_ORIGIN_ID],
         editorial_type=metas.get(SemanticMetadata.EDITORIAL_TYPE, "book"),
         lang=metas.get(FT_LANG, "en-US"),
@@ -293,14 +287,14 @@ def do_collect(
     with TemporaryFile() as ftmap_file:
         ftmap.write(ftmap_file)
         ftmap_file.seek(0)
-        _create_zip_file(output, pinput, image_dir, ftmap_file, collected_files)
+        _create_zip_file(output, pinput, image_dir, ftmap_file, collected_files, dev_mode=dev_mode)
 
 
 def do_publish(
-    zip: Path,
-    ft_session: FTSession,
-    source_id: str,
-    dry_run: bool,
+        zip: Path,
+        ft_session: FTSession,
+        source_id: str,
+        dry_run: bool,
 ):
     """Upload and publish an FTML archive to an FT portal."""
     if not dry_run:
@@ -339,11 +333,11 @@ def _get_meta_values(key: str, metadatas: List[dict], first_value_only: bool):
 
 
 def do_unpublish(
-    ft_session: FTSession,
-    source_id: str,
-    dry_run: bool,
-    do_it: bool,
-    select: str,
+        ft_session: FTSession,
+        source_id: str,
+        dry_run: bool,
+        do_it: bool,
+        select: str,
 ):
     """
     Delete selected publications by :
@@ -557,11 +551,11 @@ def add_options(options):
     default="ftml_content.zip",
 )
 def collect(
-    input: str,
-    output: str,
-    included_audiences: List[str],
-    implicit_meta: bool,
-    dev_mode: bool,
+        input: str,
+        output: str,
+        included_audiences: List[str],
+        implicit_meta: bool,
+        dev_mode: bool,
 ):
     """Collect documentation and create an FTML archive based on the metadata found in
     the Markdown files (unless implicit_meta is set as True).
@@ -577,16 +571,16 @@ def collect(
 @add_options(collect_opts)
 @add_options(publish_opts)
 def collect_and_publish(
-    input: str,
-    included_audiences: List[str],
-    implicit_meta: bool,
-    dev_mode: bool,
-    portal_base_url: str,
-    api_key: str,
-    user: str,
-    passwd: str,
-    source_id: str,
-    dry_run: bool,
+        input: str,
+        included_audiences: List[str],
+        implicit_meta: bool,
+        dev_mode: bool,
+        portal_base_url: str,
+        api_key: str,
+        user: str,
+        passwd: str,
+        source_id: str,
+        dry_run: bool,
 ):
     """Collect upload and publish documentation to an FT portal."""
     session = verify_credentials(portal_base_url, {"api_key": api_key, "user": user, "password": passwd}, dry_run)
@@ -606,13 +600,13 @@ def collect_and_publish(
 )
 @add_options(publish_opts)
 def publish(
-    zip: Path,
-    portal_base_url: str,
-    api_key: str,
-    user: str,
-    passwd: str,
-    source_id: str,
-    dry_run: bool,
+        zip: Path,
+        portal_base_url: str,
+        api_key: str,
+        user: str,
+        passwd: str,
+        source_id: str,
+        dry_run: bool,
 ):
     """Upload and publish an FTML archive to an FT portal."""
     session = verify_credentials(portal_base_url, {"api_key": api_key, "user": user, "password": passwd}, dry_run)
@@ -640,14 +634,14 @@ def publish(
     """,
 )
 def delete_book(
-    portal_base_url: str,
-    api_key: str,
-    user: str,
-    passwd: str,
-    source_id: str,
-    dry_run: bool,
-    do_it: bool,
-    select: str,
+        portal_base_url: str,
+        api_key: str,
+        user: str,
+        passwd: str,
+        source_id: str,
+        dry_run: bool,
+        do_it: bool,
+        select: str,
 ):
     """Delete selected publications from an FT portal.
 
