@@ -34,9 +34,11 @@ class FTMap:
             self,
             toc_node: etree.Element,
             metas: Dict[str, dict],
-            excluded_metas: List = ["ft:title"]
+            excluded_metas: List = None
             ) -> None:
         """Add a meta in <ft:metas> for the specified toc node"""
+        if excluded_metas is None:
+            excluded_metas = []
         metas_node = etree.SubElement(toc_node, f"{ftns}metas", nsmap=self.root.nsmap)
         for meta_name, meta_value in metas.items():
             if meta_name in excluded_metas:
@@ -63,7 +65,9 @@ class FTMap:
         """Add a node and its child to the ToC"""
         collected_files = []
         if current is not None:
-            attributes = {f"{ftns}title": current["ft:title"]}
+            attributes = {}
+            if current.get(f"ft:title", None) is not None:
+                attributes[f"{ftns}title"] = current.get("ft:title")
             if current.get("ft:originID", None) is not None:
                 attributes[f"{ftns}originID"] = current.get("ft:originID")
             else:
