@@ -1,25 +1,34 @@
+from enum import Enum
+
 import logging
 import re
-from pathlib import Path
-
-from typing import Tuple, Dict, TextIO
-from io import StringIO
-from enum import Enum
 import yaml
+from io import StringIO
+from pathlib import Path
+from typing import Tuple, Dict, TextIO, Optional
 
 from fluidtopics.connector.model.metadata import SemanticMetadata
+from fluidtopics.markdown.errors import MD2FTError
+
+MISSING_TITLE = 'MISSING TITLE'
+COVER_PAGE_TITLE = 'Preface'
 
 
 class MetadataError(Exception):
     pass
 
 
+class ContentBeforeHeading(Exception):
+    pass
+
+
 logger = logging.getLogger("fluidtopics.markdown")
 
 RE_LINEMETA = re.compile(r'\s*\[_metadata_\:(\w*[\:\w*]*)\]\:\s*-\s*["\'](.+)["\']')
 RE_YAMLMETA_BEGIN = re.compile(r'^---')
 RE_YAMLMETA_END = re.compile(r'^---|^\.\.\.')
 RE_H1_MD = re.compile("^# (.*)$")
+RE_CODE_MD = re.compile(r'^```$')
 
 
 class MetadataType(Enum):
@@ -27,6 +36,7 @@ class MetadataType(Enum):
     LineMeta = 2
     YamlMeta = 3
 
+
 # List of meta data that ought to be single-valued
 # See: https://doc.antidot.net/r/FT/3.7/empower-metadata/Metadata-in-Fluid-Topics/Semantic-Metadata
 
@@ -60,42 +70,55 @@ def _detect_metadata(f: Path) -> MetadataType:
     return metatype
 
 
-def _get_title_from_md_h1(line: str, metas: Dict[str, str]) -> bool:
-    match_h1 = RE_H1_MD.match(line)
-    if match_h1:
-        metas[SemanticMetadata.TITLE] = match_h1.group(1)
-        return True
-    return False
-
-
-def _read_yaml_metas(mdfile: TextIO, md_content: StringIO) -> Tuple[Dict[str, str], str, int]:
+def _read_yaml_metas(mdfile: TextIO, md_content: StringIO, is_root_readme: bool = False) -> Tuple[Dict[str, str], StringIO, int, str]:
     metas = {}
     # skip first line which should be the YAML data begin marker
     _ = mdfile.readline()
     yamldata = StringIO()
     buffer = yamldata
     end_marker = False
-    title_in_meta = False
     h1_count = 0
+    content_written = False
+    in_code_block = False
+    title = MISSING_TITLE
     for k, line in enumerate(mdfile.readlines()):
         if RE_YAMLMETA_END.match(line):
             metas.update(yaml.safe_load(yamldata.getvalue()))
-            title_in_meta = SemanticMetadata.TITLE in metas
             buffer = md_content
             end_marker = True
             continue
-        if not title_in_meta and _get_title_from_md_h1(line, metas):
-            h1_count += 1
-        else:
-            buffer.write(line)
+        try:
+            h1_count, content_written, in_code_block = _write_md_content(buffer, content_written, h1_count, line,
+                                                                         in_code_block, md_content == buffer)
+        except ContentBeforeHeading as e:
+            raise ContentBeforeHeading(f"Cannot process file: <{mdfile.name}>") from e
+        title = _get_heading_title(line, is_root_readme) or title
     if not end_marker:
         raise MetadataError("No yaml metadata end marker found ??")
-    return metas, md_content, h1_count
+    return metas, md_content, h1_count, title
 
 
-def _read_legacy_metas(mdfile: TextIO, md_content: StringIO) -> Tuple[Dict[str, str], str, int]:
+def _write_md_content(buffer: StringIO, is_content_written: bool, h1_count: int, line: str, is_code_block: bool,
+                      is_content: bool = True) -> Tuple[int, bool, bool]:
+    if RE_H1_MD.match(line) and not is_code_block:
+        h1_count += 1
+        if is_content_written and h1_count == 1:
+            raise ContentBeforeHeading(f"Content before first heading unsupported")
+    else:
+        if line.strip() and is_content:
+            is_content_written = True
+        if RE_CODE_MD.match(line):
+            is_code_block = not is_code_block
+        buffer.write(line)
+    return h1_count, is_content_written, is_code_block
+
+
+def _read_legacy_metas(mdfile: TextIO, md_content: StringIO, is_root_readme: bool) -> Tuple[Dict[str, str], StringIO, int, str]:
     metas = {}
     h1_count = 0
+    is_content_written = False
+    is_code_block = False
+    title = MISSING_TITLE
     for line in mdfile:
         m = RE_LINEMETA.match(line)
         if m:
@@ -113,43 +136,57 @@ def _read_legacy_metas(mdfile: TextIO, md_content: StringIO) -> Tuple[Dict[str,
                 else:
                     metas[key] = value
         else:
-            if _get_title_from_md_h1(line, metas):
-                h1_count += 1
-                continue
-            md_content.write(line)
-    return metas, md_content, h1_count
+            h1_count, is_content_written, is_code_block = _write_md_content(md_content, is_content_written, h1_count,
+                                                                            line, is_code_block)
+            title = _get_heading_title(line, is_root_readme) or title
+    return metas, md_content, h1_count, title
 
 
-def get_md_metas(f: Path, implicit_meta: bool = False) -> Tuple[Dict[str, str], str]:
+def _get_heading_title(line: str, is_root_readme: bool) -> Optional[str]:
+    heading_match = RE_H1_MD.match(line)
+    if is_root_readme:
+        return COVER_PAGE_TITLE
+    if heading_match:
+        return heading_match.group(1)
+    return None
+
+
+def get_md_metas(f: Path, implicit_meta: bool = False, is_root_readme: bool = False,
+                 dev_mode: bool = False) -> Tuple[Dict[str, str], str, str]:
     """Get markdown metadata from a file and verify other MD content is there too"""
     metatype = _detect_metadata(f)
     # at the end the md_content will contain the md data with the metadata stripped-out
     md_content = StringIO()
     h1_count = 0
-
+    is_content_written = False
+    is_code_block = False
     logger.debug(f"MD file={f}, MetadataType={metatype}")
     with f.open('r') as mdfile:
         if metatype == MetadataType.LineMeta:
-            metas, md_content, h1_count = _read_legacy_metas(mdfile, md_content)
+            metas, md_content, h1_count, title = _read_legacy_metas(mdfile, md_content, is_root_readme)
         elif metatype == MetadataType.YamlMeta:
-            metas, md_content, h1_count = _read_yaml_metas(mdfile, md_content)
+            metas, md_content, h1_count, title = _read_yaml_metas(mdfile, md_content, is_root_readme)
         elif metatype == MetadataType.NoMeta:
             # when no meta is present in file we should at least
             # handle H1 header as ft:title.
             metas = {}
+            title = MISSING_TITLE
             for line in mdfile:
-                if _get_title_from_md_h1(line, metas):
-                    h1_count += 1
-                else:
-                    md_content.write(line)
+                h1_count, is_content_written, is_code_block = _write_md_content(md_content, is_content_written,
+                                                                                h1_count, line, is_code_block)
+                title = _get_heading_title(line, is_root_readme) or title
 
     if h1_count > 1:
-        raise MetadataError("No ft:title meta and several H1 in content unsupported")
+        raise MetadataError(f'Several H1 in content is unsupported (in {f})')
 
     metas = _handle_implicit_and_legacy_metas(metas, f,
                                               metatype, implicit_meta)
     metas = _check_multivalued_metas(metas, f)
-    return metas, md_content.getvalue()
+    if SemanticMetadata.TITLE not in metas and title == MISSING_TITLE and not dev_mode:
+        raise MD2FTError(f"No ft:title in {f}.")
+    if is_root_readme:
+        metas.pop(SemanticMetadata.TITLE, None)
+    return metas, md_content.getvalue(), title
 
 
 def _handle_implicit_and_legacy_metas(metas: Dict[str, str],
